/* eslint-disable indent,no-unused-vars */
'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const http = require("http");
const loader_1 = require("./loader");
const connectionMiddlewareInit_1 = require("./connectionMiddlewareInit");
const CtxEventSymbol = Symbol.for('EGG-MQTT#CTX-EVENT');
const RouterConfigSymbol = Symbol.for('EGG-MQTT#ROUTERCONFIG');
module.exports = (app) => {
    loader_1.loader(app);
    const config = app.config.mqtt;
    const namespace = config.namespace;
    const connectionMiddlewares = new Map();
    const packetMiddlewares = new Map();
    for (const nsp in namespace) {
        const connectionMiddlewareConfig = namespace[nsp].connectionMiddleware;
        const packetMiddlewareConfig = namespace[nsp].packetMiddleware;
        const cmw = [];
        const pmw = [];
        if (connectionMiddlewareConfig) {
            for (const middleware of connectionMiddlewareConfig) {
                cmw.push(app.mqtt.middleware[middleware]);
            }
            connectionMiddlewares.set(nsp, cmw);
        }
        if (packetMiddlewareConfig) {
            for (const middleware of packetMiddlewareConfig) {
                pmw.push(app.mqtt.middleware[middleware]);
            }
            packetMiddlewares.set(nsp, pmw);
        }
    }
    const errorEvent = {
        clientDisconnected: 'clientDisconnected',
        clientError: 'clientError',
    };
    app.mqtt.on('offline', () => {
        const map = app[RouterConfigSymbol];
        if (!map) {
            return;
        }
        http.get(config.pingUrl, request => {
            const map = app[RouterConfigSymbol];
            const handler = map.get('offline');
            if (!handler)
                return;
            const ctx = app.createContext(request, new http.ServerResponse(request));
            handler.call(ctx)
                .catch(e => {
                e.message = '[egg-socket.io] controller execute error: ' + e.message;
                app.coreLogger.error(e);
            });
        });
    });
    app.mqtt.once('connect', (connack) => __awaiter(this, void 0, void 0, function* () {
        /*map.keys().forEach((key) => {
            app.mqtt.subscribe(key, (err, granted) => {
                if (err) {
                    console.log(err);
                    console.log('subscribe fuck');
                } else {
                    console.log(`${key} subscribe success`);
                }

            });
        })*/
        http.get(config.pingUrl, request => {
            const map = app[RouterConfigSymbol];
            if (!map) {
                return;
            }
            const handler = map.get('ready');
            if (!handler)
                return;
            const ctx = app.createContext(request, new http.ServerResponse(request));
            handler.call(ctx)
                .catch(e => {
                e.message = '[egg-socket.io] controller execute error: ' + e.message;
                app.logger.error(e);
            });
        }).on('error', (e) => {
            console.log(e);
        });
    }));
    app.mqtt.on('message', (topic, message, packet) => {
        if (message.toString().length === 0) {
            return;
        }
        let payloadJson = null;
        try {
            payloadJson = JSON.parse(packet.payload);
            processJson(app, connectionMiddlewares, topic, message, payloadJson);
        }
        catch (e) {
            console.log(e);
            processBuffer(topic, message, packet);
        }
    });
    app.mqtt.on('error', (error => {
        console.log(error);
    }));
    function processBuffer(topic, message, packet) {
        // console.log(packet);
        console.log('payload must be json string!!!三角轮子暂时不处理其他格式!!!!');
    }
    function processJson(app, connectionMiddlewares, topic, message, payload) {
        /*if (!('action' in payload)) {
            return;
        }
        let action = payload.action;*/
        const map = app[RouterConfigSymbol];
        if (!map) {
            return;
        }
        const handler = map.get(topic);
        if (!handler)
            return;
        http.get(config.pingUrl, request => {
            const ctx = app.createContext(request, new http.ServerResponse(request));
            ctx.args = payload;
            ctx.topic = topic;
            if (errorEvent[topic]) {
                handler.call(ctx)
                    .catch(e => {
                    e.message = '[egg-socket.io] controller execute error: ' + e.message;
                    app.coreLogger.error(e);
                });
            }
            else {
                if (connectionMiddlewares.get(topic)) {
                    connectionMiddlewareInit_1.connectionMiddlewaresInit(ctx, app.mqtt, () => {
                        handler.call(ctx)
                            // .then(() => ctx[CtxEventSymbol].emit('finshed'))
                            .catch(e => {
                            if (e instanceof Error) {
                                e.message = '[egg-mqtt] controller execute error: ' + e.message;
                            }
                            else /* istanbul ignore next */ {
                            }
                        });
                    }, connectionMiddlewares.get(topic));
                }
                else {
                    handler.call(ctx)
                        // .then(() => ctx[CtxEventSymbol].emit('finshed'))
                        .catch(e => {
                        if (e instanceof Error) {
                            app.logger.error(e);
                            e.message = '[egg-mqtt] controller execute error: ' + e.message;
                        }
                        else /* istanbul ignore next */ {
                            console.log(e);
                        }
                    });
                }
            }
        }).on('error', (e) => {
            app.logger.error(e);
        });
    }
};
