import {debug} from 'debug';
import {utils} from 'egg-core';
import {EventEmitter} from 'events';
import * as http from 'http';
import * as compose from 'koa-compose';

const CtxEventSymbol = Symbol.for('EGG-SOCKET.IO#CTX-EVENT');
const RouterConfigSymbol = Symbol.for('EGG-MQTT#ROUTERCONFIG');

export function packetMiddlewareInit(app, mosca, packet, next, packetMiddlewares, nsp) {
    const request = mosca.request;
    request.mqtt = mosca;
    const ctx = app.createContext(request, new http.ServerResponse(request));
    ctx.packet = packet;
    ctx[CtxEventSymbol] = new EventEmitter();
    packetMiddlewares = packetMiddlewares.map((mw) => utils.middleware(mw));
    const composed = compose(packetMiddlewares);

    composed(ctx, async () => {
        packet.push(ctx);
        next();
        const eventName = packet[0];
        const routerMap = nsp[RouterConfigSymbol];
        if (routerMap && routerMap.has(eventName)) {
            debug('[egg-mqtt] wait controller finshed!');
            await new Promise((resolve, reject) => {
                ctx[CtxEventSymbol].on('finshed', (error) => {
                    debug('[egg-mqtt] controller execute finished, resume middlewares');
                    if (!error) {
                        resolve();
                    }
                    else {
                        reject(error);
                    }
                });
            });
        }
    })
        .catch((e) => {
            next(e); // throw to the native socket.io
            app.coreLogger.error(e);
        });
};